@@ -12,26 +12,26 @@
 -- Computation of source terms. Used in compilation and in @cc@ command.
 -----------------------------------------------------------------------------
 
-module GF.Compile.Compute.ConcreteLazy (computeConcrete, computeTerm,checkPredefError) where
-
-import GF.Data.Operations
+module GF.Compile.Compute.ConcreteLazy ({-computeConcrete, computeTerm,checkPredefError-}) where
+{-
 import GF.Grammar.Grammar
+import GF.Data.Operations
 import GF.Infra.Ident
-import GF.Infra.Option
+--import GF.Infra.Option
 import GF.Data.Str
-import GF.Grammar.ShowTerm
+--import GF.Grammar.ShowTerm
 import GF.Grammar.Printer
 import GF.Grammar.Predef
 import GF.Grammar.Macros
 import GF.Grammar.Lookup
-import GF.Compile.Refresh
+--import GF.Compile.Refresh
 import GF.Grammar.PatternMatch
 import GF.Grammar.Lockfield (isLockLabel,unlockRecord) ----
 
 import GF.Compile.Compute.AppPredefined
 
-import Data.List (nub,intersperse)
-import Control.Monad (liftM2, liftM)
+import Data.List (nub) --intersperse
+--import Control.Monad (liftM2, liftM)
 import Control.Monad.Identity
 import Text.PrettyPrint
 
@@ -195,7 +195,8 @@ computeTermOpt gr = comput True where
 ----              foldr1 C (map K (str2strings (glueStr v (str a)))) | v <- x']
            ,return $ Glue x y
            ]
-         (C u v,_) -> comp g $ C u (Glue v y)  -- (u ++ v) + y
+         (C u v,_) -> comp g $ C u (Glue v y)    -- (u ++ v) + y
+         (_,C u v) -> comp g $ C (Glue x u) v    -- x ++ (u ++ v)
 
          _ -> do
            mapM_ checkNoArgVars [x,y]
@@ -507,8 +508,7 @@ computeTermOpt gr = comput True where
 -- | argument variables cannot be glued
 checkNoArgVars :: Term -> Comp Term
 checkNoArgVars t = case t of
-  Vr (IA _ _)    -> fail $ glueErrorMsg $ ppTerm Unqualified 0 t 
-  Vr (IAV _ _ _) -> fail $ glueErrorMsg $ ppTerm Unqualified 0 t 
+  Vr x | isArgIdent x -> fail $ glueErrorMsg $ ppTerm Unqualified 0 t
   _ -> composOp checkNoArgVars t
 
 glueErrorMsg s = 
@@ -519,12 +519,6 @@ getArgType t = case t of
   V ty _ -> return ty
   T (TComp ty) _ -> return ty
   _ -> fail (render (text "cannot get argument type of table" $$ nest 2 (ppTerm Unqualified 0 t)))
-
-checkPredefError :: SourceGrammar -> Term -> Err Term
-checkPredefError sgr t =
-    case t of
-      Error s -> fail ("Error: "++s)
-      _ -> return t
   
 {-
 -- Old
@@ -534,3 +528,4 @@ checkPredefError sgr t = case t of
   
 predef_error s = App (Q (cPredef,cError)) (K s)
 -}
+-}
