@@ -18,14 +18,14 @@ import GF.Data.Operations
 import GF.Grammar.Grammar
 import GF.Infra.Ident
 import GF.Infra.Option
-import GF.Infra.Modules
+--import GF.Infra.Modules
 import GF.Data.Str
 import GF.Grammar.ShowTerm
 import GF.Grammar.Printer
 import GF.Grammar.Predef
 import GF.Grammar.Macros
 import GF.Grammar.Lookup
-import GF.Compile.Refresh
+--import GF.Compile.Refresh
 import GF.Grammar.PatternMatch
 import GF.Grammar.Lockfield (isLockLabel,unlockRecord) ----
 
