@@ -10,46 +10,48 @@
 -----------------------------------------------------------------------------
 
 module GF.Compile.GeneratePMCFG
-    (generatePMCFG, pgfCncCat, addPMCFG
+    (generatePMCFG, pgfCncCat, addPMCFG, resourceValues
     ) where
 
-import PGF.CId
-import PGF.Data hiding (Type, Production)
+--import PGF.CId
+import PGF.Internal as PGF(CncCat(..),Symbol(..),fidVar)
 
 import GF.Infra.Option
 import GF.Grammar hiding (Env, mkRecord, mkTable)
 import GF.Grammar.Lookup
 import GF.Grammar.Predef
+import GF.Grammar.Lockfield (isLockLabel)
 import GF.Data.BacktrackM
 import GF.Data.Operations
-import GF.Data.Utilities (updateNthM, updateNth, sortNub)
-
-import System.IO
+import GF.Infra.UseIO (IOE,ePutStr,ePutStrLn)
+import GF.Data.Utilities (updateNthM) --updateNth
+import GF.Compile.Compute.ConcreteNew(GlobalEnv,normalForm,resourceValues,ppL)
 import qualified Data.Map as Map
 import qualified Data.Set as Set
 import qualified Data.List as List
-import qualified Data.IntMap as IntMap
+--import qualified Data.IntMap as IntMap
 import qualified Data.IntSet as IntSet
-import qualified Data.ByteString.Char8 as BS
 import Text.PrettyPrint hiding (Str)
 import Data.Array.IArray
 import Data.Array.Unboxed
-import Data.Maybe
-import Data.Char (isDigit)
+--import Data.Maybe
+--import Data.Char (isDigit)
 import Control.Monad
 import Control.Monad.Identity
-import Control.Exception
+--import Control.Exception
+--import Debug.Trace(trace)
 
 ----------------------------------------------------------------------
 -- main conversion function
 
-generatePMCFG :: Options -> [SourceModule] -> SourceModule -> IO SourceModule
-generatePMCFG opts mos cmo@(cm,cmi) = do
-  (seqs,js) <- mapAccumWithKeyM (addPMCFG opts gr am cm) Map.empty (jments cmi)
-  when (verbAtLeast opts Verbose) $ hPutStrLn stderr ""
+generatePMCFG :: Options -> SourceGrammar -> Maybe FilePath -> SourceModule -> IOE SourceModule
+generatePMCFG opts sgr opath cmo@(cm,cmi) = do
+  (seqs,js) <- mapAccumWithKeyM (addPMCFG opts gr cenv opath am cm) Map.empty (jments cmi)
+  when (verbAtLeast opts Verbose) $ ePutStrLn ""
   return (cm,cmi{mseqs = Just (mkSetArray seqs), jments = js})
   where
-    gr = mGrammar (cmo:mos)
+    cenv = resourceValues gr
+    gr = prependModule sgr cmo
     MTConcrete am = mtype cmi
 
 mapAccumWithKeyM :: (Monad m, Ord k) => (a -> k -> b -> m (a,c)) -> a
@@ -64,15 +66,15 @@ mapAccumWithKeyM f a m = do let xs = Map.toAscList m
                                    return (a,(k,y):kys)
 
 
-addPMCFG :: Options -> SourceGrammar -> Ident -> Ident -> SeqSet -> Ident -> Info -> IO (SeqSet, Info)
-addPMCFG opts gr am cm seqs id (GF.Grammar.CncFun mty@(Just (cat,cont,val)) mlin@(Just (L _ term)) mprn Nothing) = do
+addPMCFG :: Options -> SourceGrammar -> GlobalEnv -> Maybe FilePath -> Ident -> Ident -> SeqSet -> Ident -> Info -> IOE (SeqSet, Info)
+addPMCFG opts gr cenv opath am cm seqs id (GF.Grammar.CncFun mty@(Just (cat,cont,val)) mlin@(Just (L loc term)) mprn Nothing) = do
+--when (verbAtLeast opts Verbose) $ ePutStr ("\n+ "++showIdent id++" ...")
   let pres  = protoFCat gr res val
       pargs = [protoFCat gr (snd $ catSkeleton ty) lincat | ((_,_,ty),(_,_,lincat)) <- zip ctxt cont]
 
       pmcfgEnv0   = emptyPMCFGEnv
-
-      b          = runCnvMonad gr (unfactor term >>= convertTerm opts CNil val) (pargs,[])
-      (seqs1,b1) = addSequencesB seqs b
+  b             <- convert opts gr cenv (floc opath loc id) term (cont,val) pargs
+  let (seqs1,b1) = addSequencesB seqs b
       pmcfgEnv1  = foldBM addRule
                           pmcfgEnv0
                           (goB b1 CNil [])
@@ -85,12 +87,13 @@ addPMCFG opts gr am cm seqs id (GF.Grammar.CncFun mty@(Just (cat,cont,val)) mlin
                        !funs_cnt        = e-s+1
                    in (prods_cnt,funs_cnt)
 
-  when (verbAtLeast opts Verbose) $ hPutStr stderr ("\n+ "++showIdent id ++ " " ++ show (product (map catFactor pargs)))
+  when (verbAtLeast opts Verbose) $
+    ePutStr ("\n+ "++showIdent id++" "++show (product (map catFactor pargs)))
   seqs1 `seq` stats `seq` return ()
-  when (verbAtLeast opts Verbose) $ hPutStr stderr (" "++show stats)
+  when (verbAtLeast opts Verbose) $ ePutStr (" "++show stats)
   return (seqs1,GF.Grammar.CncFun mty mlin mprn (Just pmcfg))
   where
-    (ctxt,res,_) = err error typeForm (lookupFunType gr am id)
+    (ctxt,res,_) = err bug typeForm (lookupFunType gr am id)
 
     addRule lins (newCat', newArgs') env0 =
       let [newCat] = getFIds newCat'
@@ -98,48 +101,119 @@ addPMCFG opts gr am cm seqs id (GF.Grammar.CncFun mty@(Just (cat,cont,val)) mlin
           newArgs  = map getFIds newArgs'
       in addFunction env0 newCat fun newArgs
 
-addPMCFG opts gr am cm seqs id (GF.Grammar.CncCat mty@(Just (L _ lincat)) mdef@(Just (L _ term)) mprn Nothing) = do
-  let pres = protoFCat gr (am,id) lincat
-      parg = protoFCat gr (identW,cVar) typeStr
+addPMCFG opts gr cenv opath am cm seqs id (GF.Grammar.CncCat mty@(Just (L _ lincat)) 
+                                                             mdef@(Just (L loc1 def))
+                                                             mref@(Just (L loc2 ref))
+                                                             mprn
+                                                             Nothing) = do
+  let pcat = protoFCat gr (am,id) lincat
+      pvar = protoFCat gr (identW,cVar) typeStr
 
       pmcfgEnv0  = emptyPMCFGEnv
 
-      b          = runCnvMonad gr (unfactor term >>= convertTerm opts CNil lincat) ([parg],[])
-      (seqs1,b1) = addSequencesB seqs b
-      pmcfgEnv1  = foldBM addRule
+  let lincont    = [(Explicit, varStr, typeStr)]
+  b             <- convert opts gr cenv (floc opath loc1 id) def (lincont,lincat) [pvar]
+  let (seqs1,b1) = addSequencesB seqs b
+      pmcfgEnv1  = foldBM addLindef
                           pmcfgEnv0
                           (goB b1 CNil [])
-                          (pres,[parg])
-      pmcfg     = getPMCFG pmcfgEnv1
-  when (verbAtLeast opts Verbose) $ hPutStr stderr ("\n+ "++showIdent id++" "++show (catFactor pres))
-  seqs1 `seq` pmcfg `seq` return (seqs1,GF.Grammar.CncCat mty mdef mprn (Just pmcfg))
+                          (pcat,[pvar])
+
+  let lincont    = [(Explicit, varStr, lincat)]
+  b             <- convert opts gr cenv (floc opath loc2 id) ref (lincont,typeStr) [pcat]
+  let (seqs2,b2) = addSequencesB seqs1 b
+      pmcfgEnv2  = foldBM addLinref
+                          pmcfgEnv1
+                          (goB b2 CNil [])
+                          (pvar,[pcat])
+
+  let pmcfg     = getPMCFG pmcfgEnv2
+
+  when (verbAtLeast opts Verbose) $ ePutStr ("\n+ "++showIdent id++" "++show (catFactor pcat))
+  seqs2 `seq` pmcfg `seq` return (seqs2,GF.Grammar.CncCat mty mdef mref mprn (Just pmcfg))
   where
-    addRule lins (newCat', newArgs') env0 =
+    addLindef lins (newCat', newArgs') env0 =
       let [newCat] = getFIds newCat'
           !fun     = mkArray lins
       in addFunction env0 newCat fun [[fidVar]]
 
-addPMCFG opts gr am cm seqs id info = return (seqs, info)
+    addLinref lins (newCat', [newArg']) env0 =
+      let newArg   = getFIds newArg'
+          !fun     = mkArray lins
+      in addFunction env0 fidVar fun [newArg]
+
+addPMCFG opts gr cenv opath am cm seqs id info = return (seqs, info)
+
+floc opath loc id = maybe (L loc id) (\path->L (External path loc) id) opath
+
+convert opts gr cenv loc term ty@(_,val) pargs =
+    case term' of
+      Error s -> fail $ render $ ppL loc (text $ "Predef.error: "++s)
+      _ -> do {-when (verbAtLeast opts Verbose) $
+                ePutStrLn $
+                  "\n"++take 10000 (renderStyle style{mode=OneLineMode}
+                    (text "term:"<+>ppU 0 term $$
+                     text "eta expanded:"<+>ppU 0 eterm $$
+                     text "normalized:"<+>ppU 0 term'))--}
+              return $ runCnvMonad gr (conv term') (pargs,[])
+  where
+    conv t = convertTerm opts CNil val =<< unfactor t
+
+    eterm = expand ty term
+    term' = {-if flag optNewComp opts
+            then-} normalForm cenv loc eterm -- new evaluator
+            --else term -- old evaluator is invoked from GF.Compile.Optimize
+
+expand (context,val) = mkAbs pars . recordExpand val . flip mkApp args
+  where pars = [(Explicit,v) | v <- vars]
+        args = map Vr vars
+        vars = map (\(bt,x,t) -> x) context
+
+recordExpand :: Type -> Term -> Term
+recordExpand typ trm =
+  case typ of
+     RecType tys -> expand trm
+       where
+         n = length tys
+         expand trm =
+           case trm of
+             FV ts -> FV (map expand ts)
+             R rs | length rs==n -> trm
+             _ -> R [assign lab (P trm lab) | (lab,_) <- tys]
+     _ -> trm
 
 unfactor :: Term -> CnvMonad Term
 unfactor t = CM (\gr c -> c (unfac gr t))
   where
-    unfac gr t = 
+    unfac gr t =
       case t of
-        T (TTyped ty) [(PV x,u)] -> V ty [restore x v (unfac gr u) | v <- err error id (allParamValues gr ty)]
+        T (TTyped ty) [(PV x,u)] -> let u' = unfac gr u
+                                        vs = allparams ty
+                                    in --trace ("expand single variable table into "++show (length vs)++" branches.\n"++render (ppU 0 t)) $
+                                       V ty [restore x v u' | v <- vs]
+        T (TTyped ty) [(PW  ,u)] -> let u' = unfac gr u
+                                        vs = allparams ty
+                                    in --trace ("expand wildcard table into "++show (length vs)++ "branches.\n"++render (ppU 0 t)) $
+                                       V ty [u' | _ <- vs]
+        T (TTyped ty) _ -> -- convertTerm doesn't handle these tables
+                           ppbug $
+                           sep [text "unfactor"<+>ppU 10 t,
+                                text (show t){-,
+                                fsep (map (ppU 10) (allparams ty))-}]
         _ -> composSafeOp (unfac gr) t
       where
+        allparams ty = err bug id (allParamValues gr ty)
+
         restore x u t = case t of
                           Vr y | y == x -> u
                           _             -> composSafeOp (restore x u) t
 
-pgfCncCat :: SourceGrammar -> Type -> Int -> PGF.Data.CncCat
+pgfCncCat :: SourceGrammar -> Type -> Int -> CncCat
 pgfCncCat gr lincat index =
   let ((_,size),schema) = computeCatRange gr lincat
-  in PGF.Data.CncCat index
-                     (index+size-1)
-                     (mkArray (map (renderStyle style{mode=OneLineMode} . ppPath) 
-                                   (getStrPaths schema)))
+  in PGF.CncCat index (index+size-1)
+                      (mkArray (map (renderStyle style{mode=OneLineMode} . ppPath) 
+                                    (getStrPaths schema)))
   where
     getStrPaths :: Schema Identity s c -> [Path]
     getStrPaths = collect CNil []
@@ -210,16 +284,17 @@ choices nr path = do (args,_) <- get
                                                      values          -> let path = reversePath rpath
                                                                         in CM (\gr c s -> Case nr path [(value, updateEnv path value gr c s)
                                                                                                                     | (value,index) <- values])
+    descend schema       path              rpath = bug $ "descend "++show (schema,path,rpath)
 
     updateEnv path value gr c (args,seq) = 
       case updateNthM (restrictProtoFCat path value) nr args of
         Just args -> c value (args,seq)
-        Nothing   -> error "conflict in updateEnv"
+        Nothing   -> bug "conflict in updateEnv"
 
 -- | the argument should be a parameter type and then
 -- the function returns all possible values.
 getAllParamValues :: Type -> CnvMonad [Term]
-getAllParamValues ty = CM (\gr c -> c (err error id (allParamValues gr ty)))
+getAllParamValues ty = CM (\gr c -> c (err bug id (allParamValues gr ty)))
 
 mkRecord :: [(Label,CnvMonad (Schema Branch s c))] -> CnvMonad (Schema Branch s c)
 mkRecord xs = CM (\gr c -> foldl (\c (lbl,CM m) bs s -> c ((lbl,m gr (\v s -> Return v) s) : bs) s) (c . CRec) xs [])
@@ -240,6 +315,15 @@ data Schema b s c
   | CTbl Type [(Term, b (Schema b s c))]
   | CStr s
   | CPar c
+--deriving Show -- doesn't work
+
+instance Show s => Show (Schema b s c) where
+  showsPrec _ sch =
+      case sch of
+        CRec r   -> showString "CRec " . shows (map fst r)
+        CTbl t _ -> showString "CTbl " . showsPrec 10 t . showString " _"
+        CStr s   -> showString "CStr " . showsPrec 10 s
+        CPar c   -> showString "CPar{}"
 
 -- | Path into a term or term schema
 data Path
@@ -281,19 +365,19 @@ computeCatRange gr lincat = compute (0,1) lincat
                                                                                 _      -> let (st',t') = compute st t
                                                                                           in (st',(lbl,Identity t'))) st rs
                                in (st',CRec rs')
-    compute st (Table pt vt) = let vs        = err error id (allParamValues gr pt)
+    compute st (Table pt vt) = let vs        = err bug id (allParamValues gr pt)
                                    (st',cs') = List.mapAccumL (\st v       -> let (st',vt') = compute st vt
                                                                               in (st',(v,Identity vt'))) st vs
                                in (st',CTbl pt cs')
     compute st (Sort s)
                    | s == cStr = let (index,m) = st
                                  in ((index+1,m),CStr index)
-    compute st t               = let vs = err error id (allParamValues gr t)
+    compute st t               = let vs = err bug id (allParamValues gr t)
                                      (index,m) = st
                                  in ((index,m*length vs),CPar (m,zip vs [0..]))
 
-ppPath (CProj lbl path) = ppLabel lbl              <+> ppPath path
-ppPath (CSel  trm path) = ppTerm Unqualified 5 trm <+> ppPath path
+ppPath (CProj lbl path) = ppLabel lbl <+> ppPath path
+ppPath (CSel  trm path) = ppU 5 trm   <+> ppPath path
 ppPath CNil             = empty
 
 reversePath path = rev CNil path
@@ -321,17 +405,51 @@ convertTerm opts sel ctype (FV vars)    = do term <- variants vars
 convertTerm opts sel ctype (C t1 t2)    = do v1 <- convertTerm opts sel ctype t1
                                              v2 <- convertTerm opts sel ctype t2
                                              return (CStr (concat [s | CStr s <- [v1,v2]]))
-convertTerm opts sel ctype (K t)        = return (CStr [SymKS [t]])
+convertTerm opts sel ctype (K t)        = return (CStr [SymKS t])
 convertTerm opts sel ctype Empty        = return (CStr [])
-convertTerm opts sel ctype (Alts s alts)
-                                        = return (CStr [SymKP (strings s) [Alt (strings u) (strings v) | (u,v) <- alts]])
-                                            where
-                                              strings (K s)     = [s]
-                                              strings (C u v)   = strings u ++ strings v
-                                              strings (Strs ss) = concatMap strings ss
+convertTerm opts sel ctype (Alts s alts)= do CStr s <- convertTerm opts CNil ctype s
+                                             alts <- forM alts $ \(u,alt) -> do
+                                                            CStr u <- convertTerm opts CNil ctype u
+                                                            Strs ps <- unPatt alt
+                                                            ps     <- mapM (convertTerm opts CNil ctype) ps
+                                                            return (u,map unSym ps)
+                                             return (CStr [SymKP s alts])
+  where
+    unSym (CStr [])        = ""
+    unSym (CStr [SymKS t]) = t
+    unSym _                = ppbug $ hang (text "invalid prefix in pre expression:") 4 (ppU 0 (Alts s alts))
+
+    unPatt (EPatt p) = fmap Strs (getPatts p)
+    unPatt u         = return u
+
+    getPatts p = case p of
+      PAlt a b  -> liftM2 (++) (getPatts a) (getPatts b)
+      PString s -> return [K s]
+      PSeq a b  -> do
+        as <- getPatts a
+        bs <- getPatts b
+        return [K (s ++ t) | K s <- as, K t <- bs]
+      _ -> fail (render (text "not valid pattern in pre expression" <+> ppPatt Unqualified 0 p))
+
+convertTerm opts sel ctype (Q (m,f))
+  | m == cPredef &&
+    f == cNonExist                      = return (CStr [SymNE])
+  | m == cPredef &&
+    f == cBIND                          = return (CStr [SymBIND])
+  | m == cPredef &&
+    f == cSOFT_BIND                     = return (CStr [SymSOFT_BIND])
+
+convertTerm opts sel@(CProj l _) ctype (ExtR t1 t2@(R rs2))
+                    | l `elem` map fst rs2 = convertTerm opts sel ctype t2
+                    | otherwise            = convertTerm opts sel ctype t1
+
+convertTerm opts sel@(CProj l _) ctype (ExtR t1@(R rs1) t2)
+                    | l `elem` map fst rs1 = convertTerm opts sel ctype t1
+                    | otherwise            = convertTerm opts sel ctype t2
+
 convertTerm opts CNil ctype t           = do v <- evalTerm CNil t
                                              return (CPar v)
-convertTerm _    _    _     t           = error (render (text "convertTerm" <+> parens (ppTerm Unqualified 0 t)))
+convertTerm _    sel  _     t           = ppbug (text "convertTerm" <+> sep [parens (text (show sel)),ppU 10 t])
 
 convertArg :: Options -> Term -> Int -> Path -> CnvMonad (Value [Symbol])
 convertArg opts (RecType rs)  nr path =
@@ -358,10 +476,11 @@ convertArg opts ty nr path              = do
   return (CPar value)
 
 convertRec opts CNil (RecType rs) record =
-  mkRecord (map (\(lbl,ctype) -> (lbl,convertTerm opts CNil ctype (projectRec lbl record))) rs)
+    mkRecord [(lbl,convertTerm opts CNil ctype (proj lbl))|(lbl,ctype)<-rs]
+  where proj lbl = if isLockLabel lbl then R [] else projectRec lbl record
 convertRec opts (CProj lbl path) ctype record =
   convertTerm opts path ctype (projectRec lbl record)
-convertRec opts _ ctype _ = error ("convertRec: "++show ctype)
+convertRec opts _ ctype _ = bug ("convertRec: "++show ctype)
 
 convertTbl opts CNil (Table _ vt) pt ts = do
   vs <- getAllParamValues pt
@@ -370,9 +489,9 @@ convertTbl opts (CSel v sub_sel) ctype pt ts = do
   vs <- getAllParamValues pt
   case lookup v (zip vs ts) of
     Just t  -> convertTerm opts sub_sel ctype t
-    Nothing -> error (render (text "convertTbl:" <+> (text "missing value" <+> ppTerm Unqualified 0 v $$
-                                                      text "among" <+> vcat (map (ppTerm Unqualified 0) vs))))
-convertTbl opts _ ctype _ _ = error ("convertTbl: "++show ctype)
+    Nothing -> ppbug (text "convertTbl:" <+> (text "missing value" <+> ppU 0 v $$
+                                              text "among" <+> vcat (map (ppU 0) vs)))
+convertTbl opts _ ctype _ _ = bug ("convertTbl: "++show ctype)
 
 
 goB :: Branch (Value SeqId) -> Path -> [SeqId] -> BacktrackM Env [SeqId]
@@ -411,7 +530,7 @@ addSequencesV seqs (CRec vs)  = let !(seqs1,vs1) = mapAccumL' (\seqs (lbl,b) ->
 addSequencesV seqs (CTbl pt vs)=let !(seqs1,vs1) = mapAccumL' (\seqs (trm,b) -> let !(seqs',b') = addSequencesB seqs b
                                                                                 in (seqs',(trm,b'))) seqs vs
                                 in (seqs1,CTbl pt vs1)
-addSequencesV seqs (CStr lin) = let !(seqs1,seqid) = addSequence seqs (optimizeLin lin)
+addSequencesV seqs (CStr lin) = let !(seqs1,seqid) = addSequence seqs lin
                                 in (seqs1,CStr seqid)
 addSequencesV seqs (CPar i)   = (seqs,CPar i)
 
@@ -421,16 +540,6 @@ mapAccumL' f s (x:xs) = (s'',y:ys)
                         where !(s', y ) = f s x
                               !(s'',ys) = mapAccumL' f s' xs
 
-optimizeLin [] = []
-optimizeLin lin@(SymKS _ : _) = 
-  let (ts,lin') = getRest lin
-  in SymKS ts : optimizeLin lin'
-  where
-    getRest (SymKS ts : lin) = let (ts1,lin') = getRest lin
-                               in (ts++ts1,lin')
-    getRest             lin  = ([],lin)
-optimizeLin (sym : lin) = sym : optimizeLin lin
-
 addSequence :: SeqSet -> [Symbol] -> (SeqSet,SeqId)
 addSequence seqs lst =
   case Map.lookup seq seqs of
@@ -450,28 +559,29 @@ evalTerm CNil (App x y)    = do x <- evalTerm CNil x
                                 y <- evalTerm CNil y
                                 return (App x y)
 evalTerm path (Vr x)       = choices (getVarIndex x) path
-evalTerm path (R rs)       = case path of
-                               (CProj lbl path) -> evalTerm path (projectRec lbl rs)
-                               CNil             -> do rs <- mapM (\(lbl,(_,t)) -> do t <- evalTerm path t
-                                                                                     return (assign lbl t)) rs
-                                                      return (R rs)
+evalTerm path (R rs)       =
+    case path of
+      CProj lbl path -> evalTerm path (projectRec lbl rs)
+      CNil           -> R `fmap` mapM (\(lbl,(_,t)) -> assign lbl `fmap` evalTerm path t) rs
 evalTerm path (P term lbl) = evalTerm (CProj lbl path) term
-evalTerm path (V pt ts)    = case path of
-                               (CSel trm path) -> do vs <- getAllParamValues pt
-                                                     case lookup trm (zip vs ts) of
-                                                       Just t  -> evalTerm path t
-                                                       Nothing -> error "evalTerm: missing value"
-                               CNil             -> do ts <- mapM (evalTerm path) ts
-                                                      return (V pt ts)
+evalTerm path (V pt ts)    =
+   case path of
+     CNil          -> V pt `fmap` mapM (evalTerm path) ts
+     CSel trm path ->
+         do vs <- getAllParamValues pt
+            case lookup trm (zip vs ts) of
+              Just t  -> evalTerm path t
+              Nothing -> ppbug $ text "evalTerm: missing value:"<+>ppU 0 trm
+                                 $$ text "among:" <+>fsep (map (ppU 10) vs)
 evalTerm path (S term sel) = do v <- evalTerm CNil sel
                                 evalTerm (CSel v path) term
 evalTerm path (FV terms)   = variants terms >>= evalTerm path
 evalTerm path (EInt n)     = return (EInt n)
-evalTerm path t            = error (render (text "evalTerm" <+> parens (ppTerm Unqualified 0 t)))
+evalTerm path t            = ppbug (text "evalTerm" <+> parens (ppU 0 t))
+--evalTerm path t            = ppbug (text "evalTerm" <+> sep [parens (text (show path)),parens (text (show t))])
 
-getVarIndex (IA _ i) = i
-getVarIndex (IAV _ _ i) = i
-getVarIndex (IC s) | isDigit (BS.last s) = (read . BS.unpack . snd . BS.spanEnd isDigit) s
+getVarIndex x = maybe err id $ getArgIndex x
+  where err = bug ("getVarIndex "++show x)
 
 ----------------------------------------------------------------------
 -- GrammarEnv
@@ -496,7 +606,7 @@ getPMCFG :: PMCFGEnv -> PMCFG
 getPMCFG (PMCFGEnv prodSet funSet) =
   PMCFG (optimize prodSet) (mkSetArray funSet)
   where
-    optimize ps = Map.foldWithKey ff [] (Map.fromListWith (++) [((fid,funid),[args]) | (Production fid funid args) <- Set.toList ps])
+    optimize ps = Map.foldrWithKey ff [] (Map.fromListWith (++) [((fid,funid),[args]) | (Production fid funid args) <- Set.toList ps])
       where
         ff :: (FId,FunId) -> [[[FId]]] -> [Production] -> [Production]
         ff (fid,funid) xs prods
@@ -532,7 +642,7 @@ restrictProtoFCat path v (PFCat cat f schema) = do
     addConstraint CNil             v (CPar (m,vs)) = case lookup v vs of
                                                        Just index -> return (CPar (m,[(v,index)]))
                                                        Nothing    -> mzero
-    addConstraint CNil             v (CStr _)      = error "restrictProtoFCat: string path"
+    addConstraint CNil             v (CStr _)      = bug "restrictProtoFCat: string path"
     
     update k0 f [] = return []
     update k0 f (x@(k,Identity v):xs)
@@ -543,3 +653,8 @@ restrictProtoFCat path v (PFCat cat f schema) = do
 
 mkArray    lst = listArray (0,length lst-1) lst
 mkSetArray map = array (0,Map.size map-1) [(v,k) | (k,v) <- Map.toList map]
+
+bug msg = ppbug (text msg)
+ppbug = error . render . hang (text "Internal error in GeneratePMCFG:") 4
+
+ppU = ppTerm Unqualified
