@@ -1,6 +1,6 @@
 {-# LANGUAGE PatternGuards #-}
-module GF.Compile.TypeCheck.Concrete( checkLType, inferLType, computeLType, ppType ) where
-
+module GF.Compile.TypeCheck.Concrete( {-checkLType, inferLType, computeLType, ppType-} ) where
+{-
 import GF.Infra.CheckM
 import GF.Data.Operations
 
@@ -9,7 +9,7 @@ import GF.Grammar.Lookup
 import GF.Grammar.Predef
 import GF.Grammar.PatternMatch
 import GF.Grammar.Lockfield (isLockLabel, lockRecType, unlockRecord)
-import GF.Compile.Compute.AppPredefined
+import GF.Compile.TypeCheck.Primitives
 
 import Data.List
 import Control.Monad
@@ -23,7 +23,7 @@ computeLType gr g0 t = comp (reverse [(b,x, Vr x) | (b,x,_) <- g0] ++ g0) t
       | isPredefConstant ty     -> return ty ---- shouldn't be needed
 
     Q (m,ident) -> checkIn (text "module" <+> ppIdent m) $ do
-      ty' <- checkErr (lookupResDef gr (m,ident)) 
+      ty' <- lookupResDef gr (m,ident)
       if ty' == ty then return ty else comp g ty' --- is this necessary to test?
 
     Vr ident  -> checkLookup ident g -- never needed to compute!
@@ -50,7 +50,7 @@ computeLType gr g0 t = comp (reverse [(b,x, Vr x) | (b,x,_) <- g0] ++ g0) t
       r' <- comp g r
       s' <- comp g s
       case (r',s') of
-        (RecType rs, RecType ss) -> checkErr (plusRecType r' s') >>= comp g
+        (RecType rs, RecType ss) -> plusRecType r' s' >>= comp g
         _ -> return $ ExtR r' s'
 
     RecType fs -> do
@@ -59,7 +59,7 @@ computeLType gr g0 t = comp (reverse [(b,x, Vr x) | (b,x,_) <- g0] ++ g0) t
 
     ELincat c t -> do
       t' <- comp g t
-      checkErr $ lockRecType c t' ---- locking to be removed AR 20/6/2009
+      lockRecType c t' ---- locking to be removed AR 20/6/2009
 
     _ | ty == typeTok -> return typeStr
     _ | isPredefConstant ty -> return ty
@@ -76,9 +76,9 @@ inferLType gr g trm = case trm of
                                                 Nothing -> checkError (text "unknown in Predef:" <+> ppIdent ident)
 
    Q ident -> checks [
-     termWith trm $ checkErr (lookupResType gr ident) >>= computeLType gr g
+     termWith trm $ lookupResType gr ident >>= computeLType gr g
      ,
-     checkErr (lookupResDef gr ident) >>= inferLType gr g
+     lookupResDef gr ident >>= inferLType gr g
      ,
      checkError (text "cannot infer type of constant" <+> ppTerm Unqualified 0 trm)
      ]
@@ -88,9 +88,9 @@ inferLType gr g trm = case trm of
                                                  Nothing -> checkError (text "unknown in Predef:" <+> ppIdent ident)
 
    QC ident -> checks [
-       termWith trm $ checkErr (lookupResType gr ident) >>= computeLType gr g
+       termWith trm $ lookupResType gr ident >>= computeLType gr g
        ,
-       checkErr (lookupResDef gr ident) >>= inferLType gr g
+       lookupResDef gr ident >>= inferLType gr g
        ,
        checkError (text "cannot infer type of canonical constant" <+> ppTerm Unqualified 0 trm)
        ]
@@ -160,7 +160,8 @@ inferLType gr g trm = case trm of
          checkLType gr g trm (Table arg val)
    V arg pts -> do
      (_,val) <- checks $ map (inferLType gr g) pts
-     return (trm, Table arg val)
+--   return (trm, Table arg val) -- old, caused issue 68
+     checkLType gr g trm (Table arg val)
 
    K s  -> do
      if elem ' ' s
@@ -213,10 +214,10 @@ inferLType gr g trm = case trm of
      sT'     <- computeLType gr g sT
 
      let trm' = ExtR r' s'
-     ---- trm'    <- checkErr $ plusRecord r' s'
+     ---- trm'    <- plusRecord r' s'
      case (rT', sT') of
        (RecType rs, RecType ss) -> do
-         rt <- checkErr $ plusRecType rT' sT'
+         rt <- plusRecType rT' sT'
          checkLType gr g trm' rt ---- return (trm', rt)
        _ | rT' == typeType && sT' == typeType -> return (trm', typeType)
        _ -> checkError (text "records or record types expected in" <+> ppTerm Unqualified 0 trm)
@@ -248,7 +249,7 @@ inferLType gr g trm = case trm of
 
    ELin c trm -> do
      (trm',ty) <- inferLType gr g trm
-     ty' <- checkErr $ lockRecType c ty ---- lookup c; remove lock AR 20/6/2009
+     ty' <- lockRecType c ty ---- lookup c; remove lock AR 20/6/2009
      return $ (ELin c trm', ty') 
 
    _ -> checkError (text "cannot infer lintype of" <+> ppTerm Unqualified 0 trm)
@@ -288,7 +289,7 @@ inferLType gr g trm = case trm of
      _ -> False
 
    inferPatt p = case p of
-     PP (q,c) ps | q /= cPredef -> checkErr $ liftM valTypeCnc (lookupResType gr (q,c))
+     PP (q,c) ps | q /= cPredef -> liftM valTypeCnc (lookupResType gr (q,c))
      PAs _ p  -> inferPatt p
      PNeg p   -> inferPatt p
      PAlt p q -> checks [inferPatt p, inferPatt q]
@@ -314,20 +315,32 @@ getOverload gr g mt ot = case appForm ot of
    matchOverload f typs ttys = do
      let (tts,tys) = unzip ttys
      let vfs = lookupOverloadInstance tys typs
-     let matches = [vf | vf@((v,_),_) <- vfs, matchVal mt v]
+     let matches = [vf | vf@((_,v,_),_) <- vfs, matchVal mt v]
+     let showTypes ty = hsep (map ppType ty)
+
+    
+     let (stys,styps) = (showTypes tys, [showTypes ty | (ty,_) <- typs])
+
+     -- to avoid strange error msg e.g. in case of unmatch record extension, show whole types if needed AR 28/1/2013
+     let (stysError,stypsError) = if elem (render stys) (map render styps)
+            then (hsep (map (ppTerm Unqualified 0) tys), [hsep (map (ppTerm Unqualified 0) ty) | (ty,_) <- typs])
+            else (stys,styps)
 
      case ([vf | (vf,True) <- matches],[vf | (vf,False) <- matches]) of
-       ([(val,fun)],_) -> return (mkApp fun tts, val)
-       ([],[(val,fun)]) -> do
-         checkWarn (text "ignoring lock fields in resolving" <+> ppTerm Unqualified 0 ot) 
-         return (mkApp fun tts, val)
-       ([],[]) -> do
-         let showTypes ty = hsep (map ppType ty)
-         checkError $ text "no overload instance of" <+> ppTerm Unqualified 0 f $$
+       ([(_,val,fun)],_) -> return (mkApp fun tts, val)
+       ([],[(pre,val,fun)]) -> do
+         checkWarn $  text "ignoring lock fields in resolving" <+> ppTerm Unqualified 0 ot $$
                       text "for" $$
                       nest 2 (showTypes tys) $$
+                      text "using" $$
+                      nest 2 (showTypes pre)
+         return (mkApp fun tts, val)
+       ([],[]) -> do
+         checkError $ text "no overload instance of" <+> ppTerm Unqualified 0 f $$
+                      text "for" $$
+                      nest 2 stysError $$
                       text "among" $$
-                      nest 2 (vcat [showTypes ty | (ty,_) <- typs]) $$
+                      nest 2 (vcat stypsError) $$
                       maybe empty (\x -> text "with value type" <+> ppType x) mt
 
        (vfs1,vfs2) -> case (noProds vfs1,noProds vfs2) of
@@ -346,7 +359,7 @@ getOverload gr g mt ot = case appForm ot of
          _ -> checkError $ text "ambiguous overloading of" <+> ppTerm Unqualified 0 f <+>
                            text "for" <+> hsep (map ppType tys) $$ 
                            text "with alternatives" $$
-                           nest 2 (vcat [ppType ty | (ty,_) <- if null vfs1 then vfs2 else vfs2])
+                           nest 2 (vcat [ppType ty | (_,ty,_) <- if null vfs1 then vfs2 else vfs2])
 
    matchVal mt v = elem mt [Nothing,Just v,Just (unlocked v)]
 
@@ -356,7 +369,7 @@ getOverload gr g mt ot = case appForm ot of
    ---- TODO: accept subtypes
    ---- TODO: use a trie
    lookupOverloadInstance tys typs = 
-     [((mkFunType rest val, t),isExact) | 
+     [((pre,mkFunType rest val, t),isExact) | 
        let lt = length tys,
        (ty,(val,t)) <- typs, length ty >= lt,
        let (pre,rest) = splitAt lt ty, 
@@ -364,7 +377,7 @@ getOverload gr g mt ot = case appForm ot of
        isExact || map unlocked pre == map unlocked tys
      ]
 
-   noProds vfs = [(v,f) | (v,f) <- vfs, noProd v]
+   noProds vfs = [(v,f) | (_,v,f) <- vfs, noProd v]
 
    noProd ty = case ty of
      Prod _ _ _ _ -> False
@@ -410,7 +423,7 @@ checkLType gr g trm typ0 = do
         case allParamValues gr arg of
           Ok vs -> do
             let ps0 = map fst cs
-            ps <- checkErr $ testOvershadow ps0 vs
+            ps <- testOvershadow ps0 vs
             if null ps 
               then return () 
               else checkWarn (text "patterns never reached:" $$
@@ -419,6 +432,12 @@ checkLType gr g trm typ0 = do
         cs' <- mapM (checkCase arg val) cs
         return (T (TTyped arg) cs', typ)
       _ -> checkError $ text "table type expected for table instead of" $$ nest 2 (ppType typ)
+    V arg0 vs ->
+      case typ of
+        Table arg1 val ->
+           do arg' <- checkEqLType gr g arg0 arg1 trm
+              vs' <- map fst `fmap` sequence [checkLType gr g v val|v<-vs]
+              return (V arg' vs',typ)
 
     R r -> case typ of --- why needed? because inference may be too difficult
        RecType rr -> do
@@ -492,7 +511,7 @@ checkLType gr g trm typ0 = do
         checkLType gr g (Let (x,(Just ty,def')) body) typ
 
     ELin c tr -> do
-      tr1 <- checkErr $ unlockRecord c tr
+      tr1 <- unlockRecord c tr
       checkLType gr g tr1 typ
 
     _ -> do
@@ -528,7 +547,7 @@ pattContext :: SourceGrammar -> Context -> Type -> Patt -> Check Context
 pattContext env g typ p = case p of
   PV x -> return [(Explicit,x,typ)]
   PP (q,c) ps | q /= cPredef -> do ---- why this /=? AR 6/1/2006
-    t <- checkErr $ lookupResType env (q,c)
+    t <- lookupResType env (q,c)
     let (cont,v) = typeFormCnc t
     checkCond (text "wrong number of arguments for constructor in" <+> ppPatt Unqualified 0 p) 
               (length cont == length ps)
@@ -695,3 +714,4 @@ checkLookup x g =
   case [ty | (b,y,ty) <- g, x == y] of
     []     -> checkError (text "unknown variable" <+> ppIdent x)
     (ty:_) -> return ty
+-}
\ No newline at end of file
