@@ -4,20 +4,19 @@ import GF.Grammar hiding (Env, VGen, VApp, VRecType)
 import GF.Grammar.Lookup
 import GF.Grammar.Predef
 import GF.Grammar.Lockfield
-import GF.Compile.Compute.ConcreteNew
-import GF.Compile.Compute.AppPredefined
+import GF.Compile.Compute.ConcreteNew1
+import GF.Compile.TypeCheck.Primitives
 import GF.Infra.CheckM
-import GF.Infra.UseIO
+--import GF.Infra.UseIO
 import GF.Data.Operations
 
 import Text.PrettyPrint
 import Data.List (nub, (\\), tails)
 import qualified Data.IntMap as IntMap
-import qualified Data.ByteString.Char8 as BS
 
-import GF.Grammar.Parser
-import System.IO
-import Debug.Trace
+--import GF.Grammar.Parser
+--import System.IO
+--import Debug.Trace
 
 checkLType :: SourceGrammar -> Term -> Type -> Check (Term, Type)
 checkLType gr t ty = runTcM $ do
@@ -245,7 +244,7 @@ tcPatt gr scope (PAlt p1 p2) ty0 = do
   tcPatt gr scope p1 ty0
   tcPatt gr scope p2 ty0
   return scope
-tcPatt gr scope p ty = error ("tcPatt "++show p)
+tcPatt gr scope p ty = unimplemented ("tcPatt "++show p)
 
 
 inferRecFields gr scope rs = 
@@ -438,8 +437,8 @@ quantify gr scope t tvs ty0 = do
     bndrs _                = []
 
 allBinders :: [Ident]    -- a,b,..z, a1, b1,... z1, a2, b2,... 
-allBinders = [ identC (BS.pack [x])          | x <- ['a'..'z'] ] ++
-             [ identC (BS.pack (x : show i)) | i <- [1 :: Integer ..], x <- ['a'..'z']]
+allBinders = [ identS [x]          | x <- ['a'..'z'] ] ++
+             [ identS (x : show i) | i <- [1 :: Integer ..], x <- ['a'..'z']]
 
 
 -----------------------------------------------------------------------
@@ -458,7 +457,7 @@ data MetaValue
 type MetaStore = IntMap.IntMap MetaValue
 data TcResult a
   = TcOk   a MetaStore [Message]
-  | TcFail             [Message]
+  | TcFail             [Message] -- First msg is error, the rest are warnings?
 newtype TcM a = TcM {unTcM :: MetaStore -> [Message] -> TcResult a}
 
 instance Monad TcM where
@@ -479,10 +478,12 @@ tcError msg = TcM (\ms msgs -> TcFail (msg : msgs))
 tcWarn :: Message -> TcM ()
 tcWarn msg = TcM (\ms msgs -> TcOk () ms ((text "Warning:" <+> msg) : msgs))
 
+unimplemented str = fail ("Unimplemented: "++str)
+
 runTcM :: TcM a -> Check a
-runTcM f = Check (\ctxt msgs -> case unTcM f IntMap.empty msgs of
-                                  TcOk x _ msgs -> Success x msgs
-                                  TcFail msgs   -> Fail msgs)
+runTcM f = case unTcM f IntMap.empty [] of
+             TcOk x _ msgs -> do checkWarnings msgs; return x
+             TcFail (msg:msgs) -> do checkWarnings msgs; checkError msg
 
 newMeta :: Sigma -> TcM MetaId
 newMeta ty = TcM (\ms msgs -> 
@@ -500,7 +501,7 @@ setMeta i mv = TcM (\ms msgs -> TcOk () (IntMap.insert i mv ms) msgs)
 
 newVar :: Scope -> Ident
 newVar scope = head [x | i <- [1..], 
-                         let x = identC (BS.pack ('v':show i)), 
+                         let x = identS ('v':show i),
                          isFree scope x]
   where
     isFree []            x = True
