@@ -22,14 +22,14 @@ import GF.Data.Operations
 import GF.Data.Str
 import GF.Infra.Ident
 import GF.Grammar.Grammar
-import GF.Grammar.Values
+--import GF.Grammar.Values
 import GF.Grammar.Predef
 import GF.Grammar.Printer
 
 import Control.Monad.Identity(Identity(..))
 import qualified Data.Traversable as T(mapM)
 import Control.Monad (liftM, liftM2, liftM3)
-import Data.Char (isDigit)
+--import Data.Char (isDigit)
 import Data.List (sortBy,nub)
 import Text.PrettyPrint
 
@@ -225,6 +225,12 @@ isPredefConstant t = case t of
   Q (mod,_) | mod == cPredef || mod == cPredefAbs -> True
   _                                               -> False
 
+checkPredefError :: Monad m => Term -> m Term
+checkPredefError t =
+    case t of
+      Error s -> fail ("Error: "++s)
+      _ -> return t
+
 cnPredef :: Ident -> Term
 cnPredef f = Q (cPredef,f)
 
@@ -262,22 +268,22 @@ mkWildCases = mkCases identW
 mkFunType :: [Type] -> Type -> Type
 mkFunType tt t = mkProd [(Explicit,identW, ty) | ty <- tt] t [] -- nondep prod
 
-plusRecType :: Type -> Type -> Err Type
+--plusRecType :: Type -> Type -> Err Type
 plusRecType t1 t2 = case (t1, t2) of
   (RecType r1, RecType r2) -> case
     filter (`elem` (map fst r1)) (map fst r2) of
       [] -> return (RecType (r1 ++ r2))
-      ls -> fail $ render (text "clashing labels" <+> hsep (map ppLabel ls))
-  _ -> fail $ render (text "cannot add record types" <+> ppTerm Unqualified 0 t1 <+> text "and" <+> ppTerm Unqualified 0 t2) 
+      ls -> raise $ render (text "clashing labels" <+> hsep (map ppLabel ls))
+  _ -> raise $ render (text "cannot add record types" <+> ppTerm Unqualified 0 t1 <+> text "and" <+> ppTerm Unqualified 0 t2) 
 
-plusRecord :: Term -> Term -> Err Term
+--plusRecord :: Term -> Term -> Err Term
 plusRecord t1 t2 =
  case (t1,t2) of
    (R r1, R r2 ) -> return (R ([(l,v) | -- overshadowing of old fields
                               (l,v) <- r1, not (elem l (map fst r2)) ] ++ r2))
    (_,    FV rs) -> mapM (plusRecord t1) rs >>= return . FV
    (FV rs,_    ) -> mapM (`plusRecord` t2) rs >>= return . FV
-   _ -> fail $ render (text "cannot add records" <+> ppTerm Unqualified 0 t1 <+> text "and" <+> ppTerm Unqualified 0 t2)
+   _ -> raise $ render (text "cannot add records" <+> ppTerm Unqualified 0 t1 <+> text "and" <+> ppTerm Unqualified 0 t2)
 
 -- | default linearization type
 defLinType :: Type
@@ -444,7 +450,7 @@ strsFromTerm t = case t of
            ]
   FV ts -> mapM strsFromTerm ts >>= return . concat
   Strs ts -> mapM strsFromTerm ts >>= return . concat  
-  _ -> fail (render (text "cannot get Str from term" <+> ppTerm Unqualified 0 t))
+  _ -> raise (render (text "cannot get Str from term" <+> ppTerm Unqualified 0 t))
 
 -- | to print an Str-denoting term as a string; if the term is of wrong type, the error msg
 stringFromTerm :: Term -> String
@@ -483,6 +489,24 @@ composOp co trm =
    ImplArg t        -> liftM ImplArg (co t)
    _ -> return trm -- covers K, Vr, Cn, Sort, EPatt
 
+composSafePattOp op = runIdentity . composPattOp (return . op)
+
+composPattOp :: Monad m => (Patt -> m Patt) -> Patt -> m Patt
+composPattOp op patt =
+  case patt of
+    PC c ps         -> liftM  (PC c) (mapM op ps)
+    PP qc ps        -> liftM  (PP qc) (mapM op ps)
+    PR as           -> liftM  PR (mapPairsM op as)
+    PT ty p         -> liftM  (PT ty) (op p)
+    PAs x p         -> liftM  (PAs x) (op p)
+    PImplArg p      -> liftM  PImplArg (op p)
+    PNeg p          -> liftM  PNeg (op p)
+    PAlt p1 p2      -> liftM2 PAlt (op p1) (op p2)
+    PSeq p1 p2      -> liftM2 PSeq (op p1) (op p2)
+    PMSeq (_,p1) (_,p2) -> liftM2 PSeq (op p1) (op p2) -- information loss
+    PRep p          -> liftM  PRep (op p)
+    _               -> return patt -- covers cases without subpatterns
+
 getTableType :: TInfo -> Err Type
 getTableType i = case i of
   TTyped ty -> return ty
@@ -518,6 +542,22 @@ collectOp co trm = case trm of
   Strs tt      -> concatMap co tt
   _            -> [] -- covers K, Vr, Cn, Sort
 
+collectPattOp :: (Patt -> [a]) -> Patt -> [a]
+collectPattOp op patt =
+  case patt of
+    PC c ps         -> concatMap op ps
+    PP qc ps        -> concatMap op ps
+    PR as           -> concatMap (op.snd) as
+    PT ty p         -> op p
+    PAs x p         -> op p
+    PImplArg p      -> op p
+    PNeg p          -> op p
+    PAlt p1 p2      -> op p1++op p2
+    PSeq p1 p2      -> op p1++op p2
+    PMSeq (_,p1) (_,p2) -> op p1++op p2
+    PRep p          -> op p
+    _               -> []     -- covers cases without subpatterns
+
 -- | to find the word items in a term
 wordsInTerm :: Term -> [String]
 wordsInTerm trm = filter (not . null) $ case trm of
@@ -559,19 +599,29 @@ allDependencies ism b =
       ResOper pty pt -> [pty,pt]
       ResOverload _ tyts -> concat [[Just ty, Just tr] | (ty,tr) <- tyts]
       ResParam (Just (L loc ps)) _ -> [Just (L loc t) | (_,cont) <- ps, (_,_,t) <- cont]
-      CncCat pty _ _ _ -> [pty]
+      CncCat pty _ _ _ _ -> [pty]
       CncFun _   pt _ _ -> [pt]  ---- (Maybe (Ident,(Context,Type))
       AbsFun pty _ ptr _ -> [pty] --- ptr is def, which can be mutual
       AbsCat (Just (L loc co)) -> [Just (L loc ty) | (_,_,ty) <- co]
       _              -> []
 
-topoSortJments :: SourceModule -> Err [(Ident,Info)]
+topoSortJments :: ErrorMonad m => SourceModule -> m [(Ident,Info)]
 topoSortJments (m,mi) = do
   is <- either
-          return 
-          (\cyc -> Bad (render (text "circular definitions:" <+> fsep (map ppIdent (head cyc)))))
+          return
+          (\cyc -> raise (render (text "circular definitions:" <+> fsep (map ppIdent (head cyc)))))
           (topoTest (allDependencies (==m) (jments mi)))
   return (reverse [(i,info) | i <- is, Ok info <- [lookupTree showIdent i (jments mi)]])
+
+topoSortJments2 :: ErrorMonad m => SourceModule -> m [[(Ident,Info)]]
+topoSortJments2 (m,mi) = do
+  iss <- either
+           return
+           (\cyc -> raise (render (text "circular definitions:"
+                                   <+> fsep (map ppIdent (head cyc)))))
+           (topoTest2 (allDependencies (==m) (jments mi)))
+  return
+    [[(i,info) | i<-is,Ok info<-[lookupTree showIdent i (jments mi)]] | is<-iss]
 {-
 -- | Smart constructor for PSeq
 pSeq p1 p2 =
