@@ -1,3 +1,4 @@
+-- LANGUAGE CPP
 module GF.Infra.Option
     (
      -- * Option types
@@ -6,7 +7,7 @@ module GF.Infra.Option
      Mode(..), Phase(..), Verbosity(..), 
      OutputFormat(..), 
      SISRFormat(..), Optimization(..), CFGTransform(..), HaskellOption(..),
-     Dump(..), Recomp(..),
+     Dump(..), Pass(..), Recomp(..),
      outputFormatsExpl, 
      -- * Option parsing 
      parseOptions, parseModuleOptions, fixRelativeLibPaths,
@@ -19,7 +20,7 @@ module GF.Infra.Option
      helpMessage,
      -- * Checking specific options
      flag, cfgTransform, haskellOption, readOutputFormat,
-     isLexicalCat, isLiteralCat, renameEncoding,
+     isLexicalCat, isLiteralCat, renameEncoding, getEncoding, defaultEncoding,
      -- * Setting specific options
      setOptimization, setCFGTransform,
      -- * Convenience methods for checking options    
@@ -35,19 +36,18 @@ import GF.Infra.GetOpt
 import GF.Grammar.Predef
 --import System.Console.GetOpt
 import System.FilePath
-import System.IO
+--import System.IO
 
-import GF.Data.ErrM
+import GF.Data.Operations(Err,ErrorMonad(..),liftErr)
 
 import Data.Set (Set)
 import qualified Data.Set as Set
-import qualified Data.ByteString.Char8 as BS
-
 
+import PGF.Internal(Literal(..))
 
 usageHeader :: String
 usageHeader = unlines 
- ["Usage: gfc [OPTIONS] [FILE [...]]",
+ ["Usage: gf [OPTIONS] [FILE [...]]",
   "",
   "How each FILE is handled depends on the file name suffix:",
   "",
@@ -68,13 +68,13 @@ helpMessage = usageInfo usageHeader optDescr
 
 
 -- FIXME: do we really want multi-line errors?
-errors :: [String] -> Err a
-errors = fail . unlines
+errors :: ErrorMonad err => [String] -> err a
+errors = raise . unlines
 
 -- Types
 
 data Mode = ModeVersion | ModeHelp | ModeInteractive | ModeRun | ModeCompiler
-          | ModeServer Int{-port-}
+          | ModeServer {-port::-}Int
   deriving (Show,Eq,Ord)
 
 data Verbosity = Quiet | Normal | Verbose | Debug
@@ -85,10 +85,11 @@ data Phase = Preproc | Convert | Compile | Link
 
 data OutputFormat = FmtPGFPretty
                   | FmtJavaScript 
+                  | FmtPython 
                   | FmtHaskell 
                   | FmtProlog
-                  | FmtProlog_Abs
                   | FmtLambdaProlog
+                  | FmtByteCode
                   | FmtBNF
                   | FmtEBNF
                   | FmtRegular
@@ -130,7 +131,8 @@ data HaskellOption = HaskellNoPrefix | HaskellGADT | HaskellLexical
 data Warning = WarnMissingLincat
   deriving (Show,Eq,Ord)
 
-data Dump = DumpSource | DumpRebuild | DumpExtend | DumpRename | DumpTypeCheck | DumpRefresh | DumpOptimize | DumpCanon
+newtype Dump = Dump Pass deriving (Show,Eq,Ord)
+data Pass = Source | Rebuild | Extend | Rename | TypeCheck | Refresh | Optimize | Canon
   deriving (Show,Eq,Ord)
 
 data Recomp = AlwaysRecomp | RecompIfNewer | NeverRecomp
@@ -149,16 +151,17 @@ data Flags = Flags {
       optGFODir          :: Maybe FilePath,
       optOutputDir       :: Maybe FilePath,
       optGFLibPath       :: Maybe FilePath,
+      optDocumentRoot    :: Maybe FilePath, -- For --server mode
       optRecomp          :: Recomp,
       optProbsFile       :: Maybe FilePath,
       optRetainResource  :: Bool,
       optName            :: Maybe String,
       optPreprocessors   :: [String],
-      optEncoding        :: String,
+      optEncoding        :: Maybe String,
       optPMCFG           :: Bool,
       optOptimizations   :: Set Optimization,
       optOptimizePGF     :: Bool,
-      optMkIndexPGF      :: Bool,
+      optSplitPGF        :: Bool,
       optCFGTransforms   :: Set CFGTransform,
       optLibraryPath     :: [FilePath],
       optStartCat        :: Maybe String,
@@ -168,7 +171,10 @@ data Flags = Flags {
       optWarnings        :: [Warning],
       optDump            :: [Dump],
       optTagsOnly        :: Bool,
-      optNewComp         :: Bool
+      optHeuristicFactor :: Maybe Double,
+      optMetaProb        :: Maybe Double,
+      optMetaToknProb    :: Maybe Double{-,
+      optNewComp         :: Bool-}
     }
   deriving (Show)
 
@@ -179,17 +185,19 @@ instance Show Options where
 
 -- Option parsing
 
-parseOptions :: [String]                   -- ^ list of string arguments
-             -> Err (Options, [FilePath])
+parseOptions :: ErrorMonad err =>
+                [String]                   -- ^ list of string arguments
+             -> err (Options, [FilePath])
 parseOptions args 
   | not (null errs) = errors errs
-  | otherwise       = do opts <- liftM concatOptions $ sequence optss
+  | otherwise       = do opts <- concatOptions `fmap` liftErr (sequence optss)
                          return (opts, files)
   where
     (optss, files, errs) = getOpt RequireOrder optDescr args
 
-parseModuleOptions :: [String]                   -- ^ list of string arguments
-                   -> Err Options
+parseModuleOptions :: ErrorMonad err =>
+                      [String]                   -- ^ list of string arguments
+                   -> err Options
 parseModuleOptions args = do
   (opts,nonopts) <- parseOptions args
   if null nonopts 
@@ -203,15 +211,18 @@ fixRelativeLibPaths curr_dir lib_dir (Options o) = Options (fixPathFlags . o)
 -- Showing options
 
 -- | Pretty-print the options that are preserved in .gfo files.
-optionsGFO :: Options -> [(String,String)]
+optionsGFO :: Options -> [(String,Literal)]
 optionsGFO opts = optionsPGF opts
-      ++ [("coding", flag optEncoding opts)]
+      ++ [("coding", LStr (getEncoding opts))]
 
 -- | Pretty-print the options that are preserved in .pgf files.
-optionsPGF :: Options -> [(String,String)]
+optionsPGF :: Options -> [(String,Literal)]
 optionsPGF opts = 
-         maybe [] (\x -> [("language",x)]) (flag optSpeechLanguage opts)
-      ++ maybe [] (\x -> [("startcat",x)]) (flag optStartCat opts)
+         maybe [] (\x -> [("language",LStr x)]) (flag optSpeechLanguage opts)
+      ++ maybe [] (\x -> [("startcat",LStr x)]) (flag optStartCat opts)
+      ++ maybe [] (\x -> [("heuristic_search_factor",LFlt x)]) (flag optHeuristicFactor opts)
+      ++ maybe [] (\x -> [("meta_prob",LFlt x)]) (flag optMetaProb opts)
+      ++ maybe [] (\x -> [("meta_token_prob",LFlt x)]) (flag optMetaToknProb opts)
 
 -- Option manipulation
 
@@ -230,6 +241,10 @@ concatOptions = foldr addOptions noOptions
 modifyFlags :: (Flags -> Flags) -> Options
 modifyFlags = Options
 
+getEncoding :: Options -> String
+getEncoding = renameEncoding . maybe defaultEncoding id . flag optEncoding
+defaultEncoding = "UTF-8"
+
 -- Default options
 
 defaultFlags :: Flags
@@ -246,21 +261,18 @@ defaultFlags = Flags {
       optGFODir          = Nothing,
       optOutputDir       = Nothing,
       optGFLibPath       = Nothing,
+      optDocumentRoot    = Nothing,
       optRecomp          = RecompIfNewer,
       optProbsFile       = Nothing,
       optRetainResource  = False,
 
       optName            = Nothing,
       optPreprocessors   = [],
-      optEncoding        = "latin1",
+      optEncoding        = Nothing,
       optPMCFG           = True,
--- #ifdef CC_LAZY
---       optOptimizations   = Set.fromList [OptStem,OptCSE],
--- #else
       optOptimizations   = Set.fromList [OptStem,OptCSE,OptExpand,OptParametrize],
--- #endif
       optOptimizePGF     = False,
-      optMkIndexPGF     = False,
+      optSplitPGF        = False,
       optCFGTransforms   = Set.fromList [CFGRemoveCycles, CFGBottomUpFilter, 
                                          CFGTopDownFilter, CFGMergeIdentical],
       optLibraryPath     = [],
@@ -271,11 +283,20 @@ defaultFlags = Flags {
       optWarnings        = [],
       optDump            = [],
       optTagsOnly        = False,
-      optNewComp         = False
+      optHeuristicFactor = Nothing,
+      optMetaProb        = Nothing,
+      optMetaToknProb    = Nothing{-,
+      optNewComp         =
+#ifdef NEW_COMP
+                           True
+#else
+                           False
+#endif
+-}
     }
 
--- Option descriptions
-
+-- | Option descriptions
+{-# NOINLINE optDescr #-}
 optDescr :: [OptDescr (Err Options)]
 optDescr = 
     [
@@ -288,6 +309,8 @@ optDescr =
      Option [] ["run"] (NoArg (mode ModeRun)) "Run in interactive mode, showing output only (no other messages).",
      Option [] ["server"] (OptArg modeServer "port") $
        "Run in HTTP server mode on given port (default "++show defaultPort++").",
+     Option [] ["document-root"] (ReqArg gfDocuRoot "DIR")
+           "Overrides the default document root for --server mode.",
      Option [] ["tags"] (NoArg (set $ \o -> o{optMode = ModeCompiler, optTagsOnly = True})) "Build TAGS file and exit.",
      Option ['E'] [] (NoArg (phase Preproc)) "Stop after preprocessing (with --preproc).",
      Option ['C'] [] (NoArg (phase Convert)) "Stop after conversion to .gf.",
@@ -298,9 +321,9 @@ optDescr =
      Option [] ["gfo-dir"] (ReqArg gfoDir "DIR") "Directory to put .gfo files in (default = '.').",
      Option ['f'] ["output-format"] (ReqArg outFmt "FMT") 
         (unlines ["Output format. FMT can be one of:",
-                  "Multiple concrete: pgf (default), gar, js, prolog, ...",
-                  "Single concrete only: cf, bnf, lbnf, gsl, srgs_xml, srgs_abnf, ...",
-                  "Abstract only: haskell, prolog_abs, ..."]),
+                  "Multiple concrete: pgf (default), js, pgf_pretty, prolog, python, ...", -- gar,
+                  "Single concrete only: bnf, ebnf, fa, gsl, jsgf, regexp, slf, srgs_xml, srgs_abnf, vxml, ....", -- cf, lbnf,
+                  "Abstract only: haskell, ..."]), -- prolog_abs,
      Option [] ["sisr"] (ReqArg sisrFmt "FMT") 
         (unlines ["Include SISR tags in generated speech recognition grammars.",
                   "FMT can be one of: old, 1.0"]),
@@ -314,7 +337,7 @@ optDescr =
      Option ['D'] ["output-dir"] (ReqArg outDir "DIR") 
            "Save output files (other than .gfo files) in DIR.",
      Option [] ["gf-lib-path"] (ReqArg gfLibPath "DIR") 
-           "Overides the value of GF_LIB_PATH.",
+           "Overrides the value of GF_LIB_PATH.",
      Option [] ["src","force-recomp"] (NoArg (recomp AlwaysRecomp)) 
                  "Always recompile from source.",
      Option [] ["gfo","recomp-if-newer"] (NoArg (recomp RecompIfNewer)) 
@@ -344,20 +367,24 @@ optDescr =
                 "Select an optimization package. OPT = all | values | parametrize | none",
      Option [] ["optimize-pgf"] (NoArg (optimize_pgf True))
                 "Enable or disable global grammar optimization. This could significantly reduce the size of the final PGF file",
-     Option [] ["mk-index"] (NoArg (mkIndex True))
-                "Add an index to the pgf file",
+     Option [] ["split-pgf"] (NoArg (splitPGF True))
+                "Split the PGF into one file per language. This allows the runtime to load only individual languages",
      Option [] ["stem"] (onOff (toggleOptimize OptStem) True) "Perform stem-suffix analysis (default on).",
      Option [] ["cse"] (onOff (toggleOptimize OptCSE) True) "Perform common sub-expression elimination (default on).",
      Option [] ["cfg"] (ReqArg cfgTransform "TRANS") "Enable or disable specific CFG transformations. TRANS = merge, no-merge, bottomup, no-bottomup, ...",
-     Option [] ["new-comp"] (NoArg (set $ \o -> o{optNewComp = True})) "Use the new experimental compiler.",
-     dumpOption "source" DumpSource,
-     dumpOption "rebuild" DumpRebuild,
-     dumpOption "extend" DumpExtend,
-     dumpOption "rename" DumpRename,
-     dumpOption "tc" DumpTypeCheck,
-     dumpOption "refresh" DumpRefresh,
-     dumpOption "opt" DumpOptimize,
-     dumpOption "canon" DumpCanon
+     Option [] ["heuristic_search_factor"] (ReqArg (readDouble (\d o -> o { optHeuristicFactor = Just d })) "FACTOR") "Set the heuristic search factor for statistical parsing",
+     Option [] ["meta_prob"] (ReqArg (readDouble (\d o -> o { optMetaProb = Just d })) "PROB") "Set the probability of introducting a meta variable in the parser",
+     Option [] ["meta_token_prob"] (ReqArg (readDouble (\d o -> o { optMetaToknProb = Just d })) "PROB") "Set the probability for skipping a token in the parser",
+--     Option [] ["new-comp"] (NoArg (set $ \o -> o{optNewComp = True})) "Use the new experimental compiler.",
+--     Option [] ["old-comp"] (NoArg (set $ \o -> o{optNewComp = False})) "Use old trusty compiler.",
+     dumpOption "source" Source,
+     dumpOption "rebuild" Rebuild,
+     dumpOption "extend" Extend,
+     dumpOption "rename" Rename,
+     dumpOption "tc" TypeCheck,
+     dumpOption "refresh" Refresh,
+     dumpOption "opt" Optimize,
+     dumpOption "canon" Canon
 
     ]
  where phase       x = set $ \o -> o { optStopAfterPhase = x }
@@ -385,10 +412,11 @@ optDescr =
                          Just p  -> set $ \o -> o { optHaskellOptions = Set.insert p (optHaskellOptions o) }
                          Nothing -> fail $ "Unknown Haskell option: " ++ x
                                             ++ " Known: " ++ show (map fst haskellOptionNames)
-       literalCat  x = set $ \o -> o { optLiteralCats = foldr Set.insert (optLiteralCats o) ((map (identC . BS.pack) . splitBy (==',')) x) }
+       literalCat  x = set $ \o -> o { optLiteralCats = foldr Set.insert (optLiteralCats o) ((map identS . splitBy (==',')) x) }
        lexicalCat  x = set $ \o -> o { optLexicalCats = foldr Set.insert (optLexicalCats o) (splitBy (==',') x) }
        outDir      x = set $ \o -> o { optOutputDir = Just x }
        gfLibPath   x = set $ \o -> o { optGFLibPath = Just x }
+       gfDocuRoot  x = set $ \o -> o { optDocumentRoot = Just x }
        recomp      x = set $ \o -> o { optRecomp = x }
        probsFile   x = set $ \o -> o { optProbsFile = Just x }
 
@@ -396,7 +424,7 @@ optDescr =
        addLibDir   x = set $ \o -> o { optLibraryPath = x:optLibraryPath o }
        setLibPath  x = set $ \o -> o { optLibraryPath = splitInModuleSearchPath x }
        preproc     x = set $ \o -> o { optPreprocessors = optPreprocessors o ++ [x] }
-       coding      x = set $ \o -> o { optEncoding = x }
+       coding      x = set $ \o -> o { optEncoding = Just x }
        startcat    x = set $ \o -> o { optStartCat = Just x }
        language    x = set $ \o -> o { optSpeechLanguage = Just x }
        lexer       x = set $ \o -> o { optLexer = Just x }
@@ -409,7 +437,7 @@ optDescr =
                          Nothing -> fail $ "Unknown optimization package: " ++ x
                          
        optimize_pgf x = set $ \o -> o { optOptimizePGF = x }
-       mkIndex x = set $ \o -> o { optMkIndexPGF = x }
+       splitPGF x = set $ \o -> o { optSplitPGF = x }
 
        toggleOptimize x b = set $ setOptimization' x b
 
@@ -421,7 +449,11 @@ optDescr =
                               Nothing -> fail $ "Unknown CFG transformation: " ++ x'
                                                 ++ " Known: " ++ show (map fst cfgTransformNames)
 
-       dumpOption s d = Option [] ["dump-"++s] (NoArg (set $ \o -> o { optDump = d:optDump o})) ("Dump output of the " ++ s ++ " phase.")
+       readDouble f x = case reads x of
+                          [(d,"")] -> set $ f d
+                          _        -> fail "A floating point number is expected"
+
+       dumpOption s d = Option [] ["dump-"++s] (NoArg (set $ \o -> o { optDump = Dump d:optDump o})) ("Dump output of the " ++ s ++ " phase.")
 
        set = return . Options
 
@@ -432,10 +464,11 @@ outputFormatsExpl :: [((String,OutputFormat),String)]
 outputFormatsExpl = 
     [(("pgf_pretty",   FmtPGFPretty),"human-readable pgf"),
      (("js",           FmtJavaScript),"JavaScript (whole grammar)"),
+     (("python",       FmtPython),"Python (whole grammar)"),
      (("haskell",      FmtHaskell),"Haskell (abstract syntax)"),
      (("prolog",       FmtProlog),"Prolog (whole grammar)"),
-     (("prolog_abs",   FmtProlog_Abs),"Prolog (abstract syntax)"),
      (("lambda_prolog",FmtLambdaProlog),"LambdaProlog (abstract syntax)"),
+     (("lp_byte_code", FmtByteCode),"Bytecode for Teyjus (abstract syntax, experimental)"),
      (("bnf",          FmtBNF),"BNF (context-free grammar)"),
      (("ebnf",         FmtEBNF),"Extended BNF"),
      (("regular",      FmtRegular),"* regular grammar"),
