@@ -1,60 +1,57 @@
 {-# LANGUAGE ScopedTypeVariables, CPP #-}
+-- | GF interactive mode
 module GFI (mainGFI,mainRunGFI,mainServerGFI) where
-
-import GF.Command.Interpreter
-import GF.Command.Importing
-import GF.Command.Commands
+import Prelude hiding (putStrLn,print)
+import qualified Prelude as P(putStrLn)
+import GF.Command.Interpreter(CommandEnv(..),commands,mkCommandEnv,emptyCommandEnv,interpretCommandLine)
+--import GF.Command.Importing(importSource,importGrammar)
+import GF.Command.Commands(flags,options)
 import GF.Command.Abstract
-import GF.Command.Parse
-import GF.Data.ErrM
-import GF.Data.Operations (chunks,err)
-import GF.Grammar hiding (Ident)
+import GF.Command.Parse(readCommandLine,pCommand)
+import GF.Data.Operations (Err(..),chunks,err,raise)
+import GF.Grammar hiding (Ident,isPrefixOf)
 import GF.Grammar.Analyse
 import GF.Grammar.Parser (runP, pExp)
-import GF.Grammar.Printer (ppGrammar, ppModule)
 import GF.Grammar.ShowTerm
 import GF.Grammar.Lookup (allOpers,allOpersTo)
-import GF.Compile.Rename
-import GF.Compile.Compute.Concrete (computeConcrete,checkPredefError)
-import GF.Compile.TypeCheck.Concrete (inferLType,ppType)
-import GF.Infra.Dependencies
+import GF.Compile.Rename(renameSourceTerm)
+--import GF.Compile.Compute.Concrete (computeConcrete,checkPredefError)
+import qualified GF.Compile.Compute.ConcreteNew as CN(normalForm,resourceValues)
+import GF.Compile.TypeCheck.RConcrete as TC(inferLType,ppType)
+import GF.Infra.Dependencies(depGraph)
 import GF.Infra.CheckM
-import GF.Infra.UseIO
+import GF.Infra.UseIO(ioErrorText)
+import GF.Infra.SIO
 import GF.Infra.Option
-import GF.Infra.Ident (showIdent)
-import GF.Infra.BuildInfo (buildInfo)
 import qualified System.Console.Haskeline as Haskeline
-import GF.Text.Coding
+--import GF.Text.Coding(decodeUnicode,encodeUnicode)
 
-import GF.Compile.Coding
+--import GF.Compile.Coding(codeTerm)
 
 import PGF
-import PGF.Data
-import PGF.Macros
+import PGF.Internal(emptyPGF,abstract,funs,lookStartCat)
 
 import Data.Char
-import Data.Maybe
 import Data.List(nub,isPrefixOf,isInfixOf,partition)
 import qualified Data.Map as Map
-import qualified Data.ByteString.Char8 as BS
+--import qualified Data.ByteString.Char8 as BS
+import qualified Data.ByteString.UTF8 as UTF8(fromString)
 import qualified Text.ParserCombinators.ReadP as RP
-import System.IO
-import System.CPUTime
-import System.Directory
-import Control.Exception
+--import System.IO(utf8)
+--import System.CPUTime(getCPUTime)
+import System.Directory({-getCurrentDirectory,-}getAppUserDataDirectory)
+import Control.Exception(SomeException,fromException,evaluate,try)
 import Control.Monad
-import Data.Version
 import Text.PrettyPrint (render)
-import GF.System.Signal
+import qualified GF.System.Signal as IO(runInterruptibly)
 #ifdef SERVER_MODE
 import GFServer(server)
 #endif
-#ifdef mingw32_HOST_OS
-import System.Win32.Console
-import System.Win32.NLS
-#endif
+import GF.System.Console(changeConsoleEncoding)
 
-import Paths_gf
+import GF.Infra.BuildInfo(buildInfo)
+import Data.Version(showVersion)
+import Paths_gf(version)
 
 mainRunGFI :: Options -> [FilePath] -> IO ()
 mainRunGFI opts files = shell (beQuiet opts) files
@@ -63,15 +60,18 @@ beQuiet = addOptions (modifyFlags (\f -> f{optVerbosity=Quiet}))
 
 mainGFI :: Options -> [FilePath] -> IO ()
 mainGFI opts files = do
-  putStrLn welcome
+  P.putStrLn welcome
   shell opts files
 
-shell opts files = loop opts =<< importInEnv emptyGFEnv opts files
+shell opts files = loop opts =<< runSIO (importInEnv emptyGFEnv opts files)
 
 #ifdef SERVER_MODE
 mainServerGFI opts0 port files =
-    server port (execute1 opts) =<< importInEnv emptyGFEnv opts files
-  where opts = beQuiet opts0
+    server port root (execute1 opts)
+      =<< runSIO (importInEnv emptyGFEnv opts files)
+  where
+    root = flag optDocumentRoot opts
+    opts = beQuiet opts0
 #else
 mainServerGFI opts files =
   error "GF has not been compiled with server mode support"
@@ -84,7 +84,8 @@ loop opts gfenv = maybe (return ()) (loop opts) =<< readAndExecute1 opts gfenv
 -- | Read and execute one command, returning Just an updated environment for
 -- | the next command, or Nothing when it is time to quit
 readAndExecute1 :: Options -> GFEnv -> IO (Maybe GFEnv)
-readAndExecute1 opts gfenv = execute1 opts gfenv =<< readCommand opts gfenv
+readAndExecute1 opts gfenv =
+    runSIO . execute1 opts gfenv =<< readCommand opts gfenv
 
 -- | Read a command
 readCommand :: Options -> GFEnv -> IO String
@@ -94,7 +95,7 @@ readCommand opts gfenv0 =
       _       -> fetchCommand gfenv0
 
 -- | Optionally show how much CPU time was used to run an IO action
-optionallyShowCPUTime :: Options -> IO a -> IO a
+optionallyShowCPUTime :: Options -> SIO a -> SIO a
 optionallyShowCPUTime opts act 
   | not (verbAtLeast opts Normal) = act
   | otherwise = do t0 <- getCPUTime
@@ -115,7 +116,7 @@ loopOptNewCPU opts gfenv'
 
 -- | Execute a given command, returning Just an updated environment for
 -- | the next command, or Nothing when it is time to quit
-execute1 :: Options -> GFEnv -> String -> IO (Maybe GFEnv)
+execute1 :: Options -> GFEnv -> String -> SIO (Maybe GFEnv)
 execute1 opts gfenv0 s0 =
   interruptible $ optionallyShowCPUTime opts $
   case pwords s0 of
@@ -176,20 +177,25 @@ execute1 opts gfenv0 s0 =
         pOpts style q ("-qual"   :ws) = pOpts style            Qualified   ws
         pOpts style q             ws  = (style,q,unwords ws)
 
-        (style,q,s) = pOpts TermPrintDefault Qualified (tail (words s0))
-
-      case runP pExp (encodeUnicode utf8 s) of
+        (style,q,s) = pOpts TermPrintDefault Qualified ws
+        {-
+        (new,ws') = case ws of
+                      "-new":ws' -> (True,ws')
+                      "-old":ws' -> (False,ws')
+                      _ -> (flag optNewComp opts,ws)
+        -}
+      case runP pExp (UTF8.fromString s) of
         Left (_,msg) -> putStrLn msg
-        Right t      -> case checkComputeTerm sgr (codeTerm (decodeUnicode utf8 . BS.pack) t) of
-                          Ok  x -> putStrLn $ showTerm sgr style q x
-                          Bad s -> putStrLn $ s
+        Right t      -> putStrLn . err id (showTerm sgr style q)
+                                 . checkComputeTerm sgr
+                                 $ {-codeTerm (decodeUnicode utf8 . BS.pack)-} t
       continue gfenv
 
     show_deps ws = do
           let (os,xs) = partition (isPrefixOf "-") ws
           ops <- case xs of
              _:_ -> do
-               let ts = [t | Right t <- map (runP pExp . encodeUnicode utf8) xs]
+               let ts = [t | Right t <- map (runP pExp . UTF8.fromString) xs]
                err error (return . nub . concat) $ mapM (constantDepsTerm sgr) ts
              _   -> error "expected one or more qualified constants as argument"
           let prTerm = showTerm sgr TermPrintDefault Qualified
@@ -212,14 +218,14 @@ execute1 opts gfenv0 s0 =
           let isRaw = elem "-raw" os 
           ops <- case ts of
              _:_ -> do
-               let Right t = runP pExp (encodeUnicode utf8 (unwords ts))
+               let Right t = runP pExp (UTF8.fromString (unwords ts))
                ty <- err error return $ checkComputeTerm sgr t
                return $ allOpersTo sgr ty
              _   -> return $ allOpers sgr 
           let sigs = [(op,ty) | ((mo,op),ty,pos) <- ops]
           let printer = if isRaw 
                           then showTerm sgr TermPrintDefault Qualified
-                          else (render . GF.Compile.TypeCheck.Concrete.ppType)
+                          else (render . TC.ppType)
           let printed = [unwords [showIdent op, ":", printer ty] | (op,ty) <- sigs]
           mapM_ putStrLn [l | l <- printed, all (flip isInfixOf l) greps]
           continue gfenv
@@ -239,7 +245,7 @@ execute1 opts gfenv0 s0 =
                  [showIdent j ++ "\t" ++ show (fst k) | (j,k) <- snd sz]
         _ | elem "-save" os -> mapM_ 
                  (\ m@(i,_) -> let file = (showIdent i ++ ".gfh") in 
-                    writeFile file (render (ppModule Qualified m)) >> putStrLn ("wrote " ++ file))
+                    restricted $ writeFile file (render (ppModule Qualified m)) >> P.putStrLn ("wrote " ++ file))
                  (modules mygr)  
         _ -> putStrLn $ render $ ppGrammar mygr
       continue gfenv
@@ -253,7 +259,7 @@ execute1 opts gfenv0 s0 =
          continue gfenv
 
     eh [w] = -- Ehhh? Reads commands from a file, but does not execute them
-      do cs <- readFile w >>= return . map (interpretCommandLine env) . lines
+      do cs <- restricted (readFile w) >>= return . map (interpretCommandLine env) . lines
          continue gfenv
     eh _   = do putStrLn "eh command not parsed"
                 continue gfenv
@@ -311,20 +317,7 @@ execute1 opts gfenv0 s0 =
 
     set_encoding [c] =
       do let cod = renameEncoding c
-#ifdef mingw32_HOST_OS
-         case cod of
-           'C':'P':c -> case reads c of
-                          [(cp,"")] -> do setConsoleCP cp
-                                          setConsoleOutputCP cp
-                          _         -> return ()
-           "UTF-8"   -> do setConsoleCP 65001
-                           setConsoleOutputCP 65001
-           _         -> return ()
-#endif
-         enc <- mkTextEncoding cod
-         hSetEncoding stdin  enc
-         hSetEncoding stdout enc
-         hSetEncoding stderr enc
+         restricted $ changeConsoleEncoding cod
          continue gfenv
     set_encoding _ = putStrLn "se command not parsed" >> continue gfenv
 
@@ -332,11 +325,13 @@ execute1 opts gfenv0 s0 =
 printException e = maybe (print e) (putStrLn . ioErrorText) (fromException e)
 
 checkComputeTerm sgr t = do
-                 mo <- maybe (Bad "no source grammar in scope") return $ greatestResource sgr
+                 mo <- maybe (raise "no source grammar in scope") return $ greatestResource sgr
                  ((t,_),_) <- runCheck $ do t <- renameSourceTerm sgr mo t
                                             inferLType sgr [] t
-                 t1 <- computeConcrete sgr t
-                 checkPredefError sgr t1
+                 t1 <- {-if new
+                       then-} return (CN.normalForm (CN.resourceValues sgr) (L NoLoc identW) t)
+                       {-else computeConcrete sgr t-}
+                 checkPredefError t1
 
 fetchCommand :: GFEnv -> IO String
 fetchCommand gfenv = do
@@ -347,13 +342,13 @@ fetchCommand gfenv = do
           Haskeline.historyFile = Just path,
           Haskeline.autoAddHistory = True
         }
-  res <- runInterruptibly $ Haskeline.runInputT settings (Haskeline.getInputLine (prompt (commandenv gfenv)))
+  res <- IO.runInterruptibly $ Haskeline.runInputT settings (Haskeline.getInputLine (prompt (commandenv gfenv)))
   case res of
     Left  _        -> return ""
     Right Nothing  -> return "q"
     Right (Just s) -> return s
 
-importInEnv :: GFEnv -> Options -> [FilePath] -> IO GFEnv
+importInEnv :: GFEnv -> Options -> [FilePath] -> SIO GFEnv
 importInEnv gfenv opts files
     | flag optRetainResource opts =
         do src <- importSource (sourcegrammar gfenv) opts files
