@@ -2,14 +2,14 @@ module ExampleDemo (Environ,initial,getNext, provideExample, testThis,mkFuncWith
   where
 
 import PGF
-import System.IO
+--import System.IO
 import Data.List
-import Control.Monad
+--import Control.Monad
 import qualified Data.Map as Map
-import qualified Data.IntMap as IntMap
+--import qualified Data.IntMap as IntMap
 import qualified Data.Set as Set
 import Data.Maybe
-import System.Environment (getArgs)
+--import System.Environment (getArgs)
 import System.Random (RandomGen) --newStdGen
 
 
