@@ -2,7 +2,7 @@ module PGF.Expr(Tree, BindType(..), Expr(..), Literal(..), Patt(..), Equation(..
                 readExpr, showExpr, pExpr, pBinds, ppExpr, ppPatt, pattScope,
 
                 mkAbs,    unAbs,
-                mkApp,    unApp,
+                mkApp,    unApp, unAppForm,
                 mkStr,    unStr,
                 mkInt,    unInt,
                 mkDouble, unDouble,
@@ -23,9 +23,9 @@ import PGF.CId
 import PGF.Type
 
 import Data.Char
-import Data.Maybe
+--import Data.Maybe
 import Data.List as List
-import Data.Map as Map hiding (showTree)
+import qualified Data.Map as Map hiding (showTree)
 import Control.Monad
 import qualified Text.PrettyPrint as PP
 import qualified Text.ParserCombinators.ReadP as RP
@@ -112,13 +112,19 @@ mkApp f es = foldl EApp (EFun f) es
 
 -- | Decomposes an expression into application of function
 unApp :: Expr -> Maybe (CId,[Expr])
-unApp = extract []
+unApp e = case unAppForm e of
+  (EFun f,es) -> Just (f,es)
+  _           -> Nothing
+
+-- | Decomposes an expression into an application of a constructor such as a constant or a metavariable
+unAppForm :: Expr -> (Expr,[Expr])
+unAppForm = extract []
   where
-    extract es (EFun f)     = Just (f,es)
+    extract es f@(EFun _)   = (f,es)
     extract es (EApp e1 e2) = extract (e2:es) e1
     extract es (ETyped e ty)= extract es e
     extract es (EImplArg e) = extract es e
-    extract es _            = Nothing
+    extract es h            = (h,es)
 
 -- | Constructs an expression from string literal
 mkStr :: String -> Expr
@@ -318,22 +324,22 @@ data Value
   | VClosure Env Expr
   | VImplArg Value
 
-type Sig = ( Map.Map CId (Type,Int,Maybe [Equation],Double)    -- type and def of a fun
-           , Int -> Maybe Expr                                 -- lookup for metavariables
+type Sig = ( Map.Map CId (Type,Int,Maybe [Equation],Double,Int) -- type and def of a fun
+           , Int -> Maybe Expr                                  -- lookup for metavariables
            )
 type Env = [Value]
 
 eval :: Sig -> Env -> Expr -> Value
 eval sig env (EVar i)     = env !! i
 eval sig env (EFun f)     = case Map.lookup f (fst sig) of
-                              Just (_,a,meqs,_) -> case meqs of
-                                                     Just eqs -> if a == 0
-                                                                   then case eqs of
-                                                                          Equ [] e : _ -> eval sig [] e
-                                                                          _            -> VConst f []
-                                                                   else VApp f []
-                                                     Nothing  -> VApp f []
-                              Nothing           -> error ("unknown function "++showCId f)
+                              Just (_,a,meqs,_,_) -> case meqs of
+                                                       Just eqs -> if a == 0
+                                                                     then case eqs of
+                                                                            Equ [] e : _ -> eval sig [] e
+                                                                            _            -> VConst f []
+                                                                     else VApp f []
+                                                       Nothing  -> VApp f []
+                              Nothing             -> error ("unknown function "++showCId f)
 eval sig env (EApp e1 e2) = apply sig env e1 [eval sig env e2]
 eval sig env (EAbs b x e) = VClosure env (EAbs b x e)
 eval sig env (EMeta i)    = case snd sig i of
@@ -347,11 +353,11 @@ apply :: Sig -> Env -> Expr -> [Value] -> Value
 apply sig env e            []     = eval sig env e
 apply sig env (EVar i)     vs     = applyValue sig (env !! i) vs
 apply sig env (EFun f)     vs     = case Map.lookup f (fst sig) of
-                                      Just (_,a,meqs,_) -> case meqs of
-                                                             Just eqs -> if a <= length vs
-                                                                           then match sig f eqs vs
-                                                                           else VApp f vs
-                                                             Nothing  -> VApp f vs
+                                      Just (_,a,meqs,_,_) -> case meqs of
+                                                               Just eqs -> if a <= length vs
+                                                                             then match sig f eqs vs
+                                                                             else VApp f vs
+                                                               Nothing  -> VApp f vs
                                       Nothing           -> error ("unknown function "++showCId f)
 apply sig env (EApp e1 e2) vs     = apply sig env e1 (eval sig env e2 : vs)
 apply sig env (EAbs b x e) (v:vs) = case (b,v) of
