@@ -30,7 +30,7 @@ module PGF.TypeCheck ( checkType, checkExpr, inferExpr
 import PGF.Data
 import PGF.Expr hiding (eval, apply, applyValue, value2expr)
 import qualified PGF.Expr as Expr
-import PGF.Macros (typeOfHypo, cidInt, cidFloat, cidString)
+import PGF.Macros (cidInt, cidFloat, cidString) -- typeOfHypo
 import PGF.CId
 
 import Data.Map as Map
@@ -38,7 +38,7 @@ import Data.IntMap as IntMap
 import Data.Maybe as Maybe
 import Data.List as List
 import Control.Monad
-import Control.Monad.Identity
+--import Control.Monad.Identity
 import Control.Monad.State
 import Control.Monad.Error
 import Text.PrettyPrint
@@ -121,13 +121,13 @@ runTcM abstr f ms s = unTcM f abstr (\x ms s cp b -> let (es,xs) = cp b
 
 lookupCatHyps :: CId -> TcM s [Hypo]
 lookupCatHyps cat = TcM (\abstr k h ms -> case Map.lookup cat (cats abstr) of
-                                            Just (hyps,_) -> k hyps ms
-                                            Nothing       -> h (UnknownCat cat))
+                                            Just (hyps,_,_,_) -> k hyps ms
+                                            Nothing           -> h (UnknownCat cat))
 
 lookupFunType :: CId -> TcM s Type
 lookupFunType fun = TcM (\abstr k h ms -> case Map.lookup fun (funs abstr) of
-                                            Just (ty,_,_,_) -> k ty ms
-                                            Nothing         -> h (UnknownFun fun))
+                                            Just (ty,_,_,_,_) -> k ty ms
+                                            Nothing           -> h (UnknownFun fun))
 
 typeGenerators :: Scope -> CId -> TcM s [(Double,Expr,TType)]
 typeGenerators scope cat = fmap normalize (liftM2 (++) x y)
@@ -143,8 +143,8 @@ typeGenerators scope cat = fmap normalize (liftM2 (++) x y)
       | cat == cidString = return [(1.0,ELit (LStr "Foo"),TTyp [] (DTyp [] cat []))]
       | otherwise        = TcM (\abstr k h ms ->
                                     case Map.lookup cat (cats abstr) of
-                                      Just (_,fns) -> unTcM (mapM helper fns) abstr k h ms
-                                      Nothing      -> h (UnknownCat cat))
+                                      Just (_,fns,_,_) -> unTcM (mapM helper fns) abstr k h ms
+                                      Nothing          -> h (UnknownCat cat))
 
     helper (p,fn) = do
       ty <- lookupFunType fn
