@@ -1,14 +1,16 @@
-module Cache (Cache,newCache,readCache) where
+module Cache (Cache,newCache,flushCache,listCache,readCache,readCache') where
 
 import Control.Concurrent.MVar
 import Data.Map (Map)
 import qualified Data.Map as Map 
 import System.Directory (getModificationTime)
-import System.Time (ClockTime)
+import System.Mem(performGC)
+import Data.Time (UTCTime)
+import Data.Time.Compat (toUTCTime)
 
 data Cache a = Cache {
       cacheLoad :: FilePath -> IO a,
-      cacheObjects :: MVar (Map FilePath (MVar (Maybe (ClockTime, a))))
+      cacheObjects :: MVar (Map FilePath (MVar (Maybe (UTCTime, a))))
     }
 
 newCache :: (FilePath -> IO a) -> IO (Cache a)
@@ -16,8 +18,18 @@ newCache load =
     do objs <- newMVar Map.empty
        return $ Cache { cacheLoad = load, cacheObjects = objs }
 
+flushCache :: Cache a -> IO ()
+flushCache c = do modifyMVar_ (cacheObjects c) (const (return Map.empty))
+                  performGC
+
+listCache :: Cache a -> IO [FilePath]
+listCache = fmap Map.keys . readMVar . cacheObjects
+
 readCache :: Cache a -> FilePath -> IO a
-readCache c file = 
+readCache c file = snd `fmap` readCache' c file
+
+readCache' :: Cache a -> FilePath -> IO (UTCTime,a)
+readCache' c file =
     do v <- modifyMVar (cacheObjects c) findEntry
        modifyMVar v readObject
   where
@@ -27,8 +39,8 @@ readCache c file =
                        Nothing -> do v <- newMVar Nothing
                                      return (Map.insert file v objs, v)
     -- Check time stamp, and reload if different than the cache entry
-    readObject m = do t' <- getModificationTime file
+    readObject m = do t' <- toUTCTime `fmap` getModificationTime file
                       x' <- case m of
                         Just (t,x) | t' == t -> return x
                         _                    -> cacheLoad c file
-                      return (Just (t',x'), x')
+                      return (Just (t',x'), (t',x'))
