@@ -1,4 +1,4 @@
-// minibar_demo.js, assumes that minibar.js and pgf_online.js have been loaded.
+// minibar_online.js, assumes that minibar.js and pgf_online.js have been loaded.
 
 var online_options={
   //grammars_url: "http://www.grammaticalframework.org/grammars/",
@@ -8,12 +8,15 @@ var online_options={
 }
 
 var minibar_options= {
-  show_abstract: true,
-  show_trees: true,
-  show_grouped_translations: false,
-  default_source_language: "Eng",
-//feedback_url: "feedback.html",
-  try_google: true
+    show_abstract: true,
+    show_trees: true,
+    tree_img_format: "png", // or "svg"
+    show_grouped_translations: false,
+    show_brackets: true,
+    word_replacements: true,
+    default_source_language: "Eng",
+    //feedback_url: "feedback.html",
+    try_google: true
 }
 
 if(/^\?\/tmp\//.test(location.search)) {
@@ -21,6 +24,12 @@ if(/^\?\/tmp\//.test(location.search)) {
     if(args[0]) online_options.grammars_url=args[0];
     if(args[1]) minibar_options.initial_grammar=args[1];
 }
+else if(supports_html5_storage()) {
+    var s=window.localStorage["gf.editor.simple.grammardir"]
+    if(s) var editor_dir=JSON.parse(s);
+}
 
 var server=pgf_online(online_options);
+if(editor_dir) server.add_grammars_url(editor_dir+"/");
+
 var minibar=new Minibar(server,minibar_options);
